//+------------------------------------------------------------------+
//|                                          AlexSTAL_ZigZagProf.mq4 |
//|                                                   AlexSTAL & PPC |
//|                      Professional Zig-Zag                        |
//+------------------------------------------------------------------+
#property copyright "Aleksandr Chugunov & George Tischenko"

#include <AlexSTAL_OutsideBar.mqh>

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Red
#property indicator_color2 Red

//     
//     2
extern int ExtPeriod = 2;
//         (  )
extern int MinAmplitude = 670;
//           
extern int MinMotion = 0;
//        High/Low 
extern bool UseSmallerTFforEB = true;

//  
double UP[], DN[];

//     
double OB[];

//     
datetime LastBarTime;
//    
int LastBarsCount, LastBarNum;
//   
double LastBarLastHigh, LastBarLastLow;
//    (    )
datetime TimeFirstExtBar;

//     
double MP, MM;

//  
bool DownloadHistory;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   if (ExtPeriod < 2)
      ExtPeriod = 2;
   
   if (MinAmplitude < 0)
      MinAmplitude = 0;
   MP = NormalizeDouble(MinAmplitude * Point, Digits);
   
   if (MinMotion < 1)
      MinMotion = 1;
   MM = NormalizeDouble(MinMotion * Point, Digits);
   
   //---- indicators
   IndicatorDigits(Digits);
   IndicatorBuffers(3);

   SetIndexBuffer(0, UP);
   SetIndexStyle(0, DRAW_ZIGZAG);
   SetIndexBuffer(1, DN);
   SetIndexStyle(1, DRAW_ZIGZAG);
   
   SetIndexBuffer(2, OB);
   
   LastBarsCount = 0;
   DownloadHistory = true;
   
   //----
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   //----
   return(0);
}
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   //      
   // ----------------------------------------------
   int counted_bars = IndicatorCounted(), i;
   
   //       
   if ( counted_bars != 0 )
      for (i = (Bars-counted_bars-2); i >= 0; i--)
        {
         if (UP[i] != EMPTY_VALUE)
            UP[i] = EMPTY_VALUE;
         if (DN[i] != EMPTY_VALUE)
            DN[i] = EMPTY_VALUE;
         if (OB[i] != EMPTY_VALUE)
            OB[i] = EMPTY_VALUE;
        }
   
   // ...     
   //    MT4 ...
   if (LastBarsCount == 0)
      LastBarsCount = Bars;
   if (Bars < LastBarsCount)
      DownloadHistory = true;
   LastBarsCount = Bars;
   
   // IndicatorCounted()     
   if ( counted_bars == 0 )
      DownloadHistory = true;
   
   //       
   if ( (counted_bars != 0) && (LastBarNum != 0) )
      if ( (Bars - iBarShift(NULL, 0, LastBarTime, true)) != LastBarNum )
         DownloadHistory = true;
   
   //  
   if (DownloadHistory)
     {
      ArrayInitialize(UP, EMPTY_VALUE);
      ArrayInitialize(DN, EMPTY_VALUE);
      ArrayInitialize(OB, EMPTY_VALUE);
      TimeFirstExtBar = 0;
      counted_bars = 0;
      LastBarTime = 0;
      LastBarNum = 0;
      DownloadHistory = false;
     }
   
   //      (  )
   bool NewBar = false;
   if (LastBarTime != Time[0])
     {
      NewBar = true;
      LastBarTime = Time[0];
      LastBarNum = Bars;
      //   -     
      LastBarLastHigh = High[0];
      LastBarLastLow = Low[0];
     }
   
   //      
   int BarsForRecalculation;
   if ( counted_bars != 0 )
     {
      BarsForRecalculation = Bars - counted_bars - 1;
      //   
      if (!NewBar)
        {
         if ( (NormalizeDouble(High[0]-LastBarLastHigh, Digits) >= MM) || (NormalizeDouble(LastBarLastLow-Low[0], Digits) >= MM) )
           {
            LastBarLastHigh = High[0];
            LastBarLastLow = Low[0];
           } else {
            //       ,     
            //           ,
            //    MinMotion
            return(0);
           }
        }
     } else {
      BarsForRecalculation = Bars - ExtPeriod;
     }
   
   //======================================================
   //========   ===============================
   //======================================================
   int LET;
   double H, L, Fup, Fdn;
   int lastUPbar, lastDNbar;
   double lastUP, lastDN;
   int m, n; //    
   for (i = BarsForRecalculation; i >= 0; i--)
     {
      //   
      // ---------------------------
      lastUP = 0;
      lastDN = 0;
      lastUPbar = i;
      lastDNbar = i;
      LET = 0; 
      m = 0; n = 0;
      while ( UP[lastUPbar] == EMPTY_VALUE )
        {
         if ( lastUPbar > (Bars-ExtPeriod) )
            break;
         lastUPbar++;
        }
      lastUP = UP[lastUPbar]; //   
      while ( DN[lastDNbar] == EMPTY_VALUE)
        {
         if ( lastDNbar > (Bars-ExtPeriod) )
            break;
         lastDNbar++;
        } 
      lastDN = DN[lastDNbar]; //   
   
      if ( lastUPbar < lastDNbar)
         LET = 1;
      if ( lastUPbar > lastDNbar)
         LET = -1;
      if ( lastUPbar == lastDNbar)
        {
         //lastUPbar==lastDNbar  ,     :
         m = lastUPbar;
         n = m;
         while ( m == n )
           {
            m++; n++;
            while ( UP[m] == EMPTY_VALUE )
              {
               if ( m > (Bars-ExtPeriod) )
                  break;
               m++;
              } //   
            while ( DN[n] == EMPTY_VALUE )
              {
               if ( n > (Bars-ExtPeriod) )
                  break;
               n++;
              } //   
            if ( MathMax(m, n) > (Bars-ExtPeriod) )
               break;
           }
         if (m < n)
            LET = 1;       //  - 
         else
            if (m > n)
               LET = -1; //  - 
        }
      //  LET==0 -             2 
      //    
      // ----------------------------------

      //----      :       
      H = High[iHighest(NULL, 0, MODE_HIGH, ExtPeriod, i)];
      L = Low[iLowest(NULL, 0, MODE_LOW, ExtPeriod, i)];
      Fup = High[i];
      Fdn = Low[i];
      
      //----        : 
      switch(Comb(i,H,L,Fup,Fdn))
        {
         //----      (Comb)      
         case 1 :
            switch(LET)
              {
               case 1 :
                  //   ,  :
                  if ( lastUP < Fup )
                    {
                     UP[lastUPbar] = EMPTY_VALUE;
                     UP[i] = Fup;
                    }
                  break;
               case -1 :
                  if ( (Fup-lastDN) > MP ) //  - 
                     UP[i] = Fup;
                  break; 
               default :
                  UP[i] = Fup;
                  TimeFirstExtBar = iTime(NULL, 0, i); //0 -     
                  break; 
              }
            break;

         //----       (Comb)          
         case -1 :
            switch(LET)
              {
               case 1 :
                  if ( (lastUP-Fdn) > MP ) //  - 
                     DN[i] = Fdn;
                  break; 
               case -1 :
                  //   ,  :
                  if ( lastDN > Fdn )
                    {
                     DN[lastDNbar] = EMPTY_VALUE;
                     DN[i] = Fdn;
                    }
                  break;
               default :
                  DN[i] = Fdn;
                  TimeFirstExtBar = iTime(NULL, 0, i); //0 -     
                  break;
              }
            break;

         //----         (Comb)        
         case 2 : //   LOW  HIGH ( )
            switch(LET)
              {
               case 1 : //  - 
                  if ( (Fup-Fdn) > MP )
                    {
                     if ( (lastUP-Fdn) > MP )
                       {
                        UP[i] = Fup;
                        DN[i] = Fdn;
                       } else {
                        if ( lastUP < Fup )
                          {
                           UP[lastUPbar] = EMPTY_VALUE;
                           UP[i] = Fup;
                          }
                       }
                    } else {
                     if ( (lastUP-Fdn) > MP )
                        DN[i] = Fdn;
                     else {
                        if ( lastUP < Fup )
                          {
                           UP[lastUPbar] = EMPTY_VALUE;
                           UP[i] = Fup;
                          }
                       }
                    }
                  break;
               case -1 : //  - 
                  if ( (Fup-Fdn) > MP )
                    {
                     UP[i] = Fup;
                     if ( (Fdn < lastDN) && (iTime(NULL, 0, lastDNbar) > TimeFirstExtBar) )
                       {
                        DN[lastDNbar] = EMPTY_VALUE;
                        DN[i] = Fdn;
                       }
                    } else {
                     if ( (Fup-lastDN) > MP )
                        UP[i] = Fup;
                     else {
                        if ( lastDN > Fdn )
                          {
                           DN[lastDNbar] = EMPTY_VALUE;
                           DN[i] = Fdn;
                          }
                       }
                    }
                  break;
               default: break;
              } //switch LET
            break;

         //----         (Comb)        
         case -2 : //   HIGH  LOW ( )
            switch(LET)
              {
               case 1 : //  - 
                  if ( (Fup-Fdn) > MP )
                    {
                     DN[i] = Fdn;
                     if ( (lastUP < Fup) && (iTime(NULL, 0, lastUPbar) > TimeFirstExtBar) )
                       {
                        UP[lastUPbar] = EMPTY_VALUE;
                        UP[i] = Fup;
                       }
                    } else {
                     if ( (lastUP-Fdn) > MP)
                        DN[i] = Fdn;
                     else {
                        if ( lastUP < Fup )
                          {
                           UP[lastUPbar] = EMPTY_VALUE;
                           UP[i] = Fup;
                          }
                       }
                    }
                  break;
               case -1 : //  - 
                  if ( (Fup-Fdn) > MP )
                    {
                     if ( (Fup-lastDN) > MP )
                       {
                        UP[i] = Fup;
                        DN[i] = Fdn;
                       } else {
                        if (lastDN > Fdn)
                          {
                           DN[lastDNbar] = EMPTY_VALUE;
                           DN[i] = Fdn;
                          }
                       }
                    } else {
                     if ( (Fup-lastDN) > MP )
                        UP[i] = Fup;
                     else {
                        if ( lastDN > Fdn)
                          {
                           DN[lastDNbar] = EMPTY_VALUE;
                           DN[i] = Fdn;
                          }
                       }
                    }
                  break;
               default: break;
              } //switch LET
            break;
         
         default: break;
        } // switch ()
     } // for ()
   //----
   return(0);
}
//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+ 
int Comb(int i, double H, double L, double Fup, double Fdn)
{
   //----
   if (Fup==H && (Fdn==0 || (Fdn>0 && Fdn>L))) return(1);  //    
   if (Fdn==L && (Fup==0 || (Fup>0 && Fup<H))) return(-1); //    
   if (Fdn==L && Fup==H)                                   //        
     {
      //       OB[]
      int OrderFormationHL = OB[i];
      if (OrderFormationHL == EMPTY_VALUE)
        {
         OrderFormationHL = GetOrderFormationBarHighLow(Symbol(), Period(), i, UseSmallerTFforEB);
         OB[i] = OrderFormationHL;
        }
      switch(OrderFormationHL)
        {
         case -1:       //   LOW  HIGH ( )
            return(2);
            break;
         case 1:        //   HIGH  LOW ( )
            return(-2); 
            break;
        }
     }
   //----  
   return(0);           //   ...
}